/*
    @author: doreshnikov
    Daniil Oreshnikov, ITMO University
*/

#include <bits/stdc++.h>

using namespace std;

typedef long long ll;
typedef unsigned int uint;
typedef pair<int, int> pii;

template<typename T1, typename T2>
istream &operator>>(istream &in, pair<T1, T2> &p) {
    return in >> p.first >> p.second;
}

template<typename T>
istream &operator>>(istream &in, vector<T> &v) {
    for (auto &item : v)
        in >> item;
    return in;
}

void mcin() {}

template<typename T, typename ...Tail>
void mcin(T &&arg, Tail &&...args) {
    cin >> arg;
    mcin(args...);
}
#define in(type, ...) type __VA_ARGS__; mcin(__VA_ARGS__);

#define all(c) c.begin(), c.end()

int const THR = 50;
using dp_t = vector<vector<int>>;

int n, q;
vector<int> a;
dp_t lcp;
vector<pair<dp_t, dp_t>> dp_all;

void naive(int l, int r) {
    cout << r - l + 1 << '\n';
    for (int i = l; i < r; i++) {
        cout << a[i] << ' ';
    }
    cout << "0\n";
}

pair<dp_t, dp_t> calc(int l, int r = n) {
    int len = r - l;
    dp_t dp(len + 1, vector<int>(len + 1, len + 1));
    dp_t back(len + 1, vector<int>(len + 1, -1));

    for (int i = 0; i <= len; i++) {
        dp[0][i] = i + 1;
        back[0][i] = 0;
    }

    for (int i = 0; i <= len; i++) {
        for (int prv = 0; prv < i; prv++) {
            int common = min({i - prv, len - i, lcp[l + prv][l + i]});
            int result = dp[prv][i] + (i - prv - common) + 1;
            if (dp[i][i + common] > result) {
                dp[i][i + common] = result;
                back[i][i + common] = prv;
            }
        }

        for (int nxt = i; nxt < len; nxt++) {
            if (dp[i][nxt + 1] > dp[i][nxt] + 1) {
                dp[i][nxt + 1] = dp[i][nxt] + 1;
                back[i][nxt + 1] = back[i][nxt];
            }
        }

        for (int nxt = len; nxt > i; nxt--) {
            if (dp[i][nxt - 1] > dp[i][nxt] + 1) {
                dp[i][nxt - 1] = dp[i][nxt] + 1;
                back[i][nxt - 1] = back[i][nxt];
            }
        }
    }

    return {dp, back};
}

pair<int, vector<int>> trace(dp_t const &dp, dp_t const &back, int l, int r) {
    int len = r - l;
    int answer = len + 1;
    int prv = 0;

    for (int i = 1; i < len; i++) {
        if (answer > dp[i][len]) {
            answer = dp[i][len];
            prv = i;
        }
    }

    int cur = len;
    vector<int> segments{cur, prv};
    while (prv != 0) {
        int tmp = prv;
        prv = back[prv][cur];
        cur = tmp;
        segments.push_back(prv);
    }

    reverse(all(segments));
    return {answer, segments};
}

void restore(vector<int> const &segments, vector<int> &actions, int l, int skip = 0) {
    vector<int> stk;
    int t = 0;
    for (int i = 1; i < segments.size(); i++) {
        int prv = segments[i - 1], cur = segments[i];
        while (stk.size() > cur - prv || i > 1 && stk.size() > lcp[l + prv][l + segments[i - 2]]) {
            actions.push_back(-1);
            stk.pop_back();
        }
        while (stk.size() < cur - prv) {
            int item = a[l + prv + stk.size()];
            if (t++ >= skip) actions.push_back(item);
            stk.push_back(item);
        }
        actions.push_back(0);
    }
}

int main() {

    ios_base::sync_with_stdio(false);
    cin.tie(nullptr);
    cout.tie(nullptr);

    cin >> n >> q;
    in(int, pts, gamma)
    a.resize(n);
    cin >> a;

    lcp.resize(n + 1, vector<int>(n + 1, 0));
    for (int i = n - 1; i > -1; i--) {
        for (int j = n - 1; j > -1; j--) {
            lcp[i][j] = a[i] == a[j] ? lcp[i + 1][j + 1] + 1 : 0;
        }
    }

    dp_all.resize(n / THR + 1);
    for (int start = 0; start < dp_all.size(); start++) {
        dp_all[start] = calc(start * THR);
    }

    for (int t = 0; t < q; t++) {
        in(int, l, r)
        l--;

        vector<int> actions;
        if (r - l <= THR) {
            auto [dp, back] = calc(l, r);
            auto [answer, segments] = trace(dp, back, l, r);
            restore(segments, actions, l);
            assert(actions.size() == answer);
        } else {
            int m = (l + THR - 1) / THR, cur = m * THR, prv = m * THR;
            if (cur > l) {
                auto [dp, back] = calc(l, cur);
                auto [_, segments] = trace(dp, back, 0, cur - l);
                restore(segments, actions, l);
                prv = l + segments[segments.size() - 2];
            }
            auto const &[dp, back] = dp_all[m];
            auto [_, segments] = trace(dp, back, 0, r - cur);
            int skip = 0;
            if (cur > l) {
                skip = min({cur - prv, segments[1] - segments[0], lcp[cur][prv]});
                for (int i = 0; i < cur - prv - skip; i++) {
                    actions.push_back(-1);
                }
            }
            restore(segments, actions, cur, skip);
        }

        if (actions.size() > r - l + 1) {
            naive(l, r);
        } else {
            cout << actions.size() << '\n';
            for (int item : actions) cout << item << ' ';
            cout << '\n';
        }
    }

}