/*
    @author: doreshnikov
    Daniil Oreshnikov, ITMO University
*/

#include <bits/stdc++.h>

using namespace std;

typedef long long ll;
typedef unsigned int uint;
typedef pair<int, int> pii;

template<typename T1, typename T2>
istream &operator>>(istream &in, pair<T1, T2> &p) {
    return in >> p.first >> p.second;
}

template<typename T>
istream &operator>>(istream &in, vector<T> &v) {
    for (auto &item : v)
        in >> item;
    return in;
}

void mcin() {}

template<typename T, typename ...Tail>
void mcin(T &&arg, Tail &&...args) {
    cin >> arg;
    mcin(args...);
}
#define in(type, ...) type __VA_ARGS__; mcin(__VA_ARGS__);

#define all(c) c.begin(), c.end()

int const THR = 50;
int const MAX_N = 2000;
using dp_t = vector<vector<int>>;

int n, q;
vector<int> a;
dp_t lcp;
vector<pair<dp_t, dp_t>> dp_all;

void naive(int l, int r) {
    cout << r - l + 1 << '\n';
    for (int i = l; i < r; i++) {
        cout << a[i] << ' ';
    }
    cout << "0\n";
}

pair<dp_t, dp_t> calc(int l, int r = n) {
    int len = r - l;
    dp_t dp(len + 1, vector<int>(len + 1, len + 1));
    dp_t back(len + 1, vector<int>(len + 1, -1));

    for (int i = 0; i <= len; i++) {
        dp[0][i] = i + 1;
        back[0][i] = 0;
    }

    for (int i = 0; i <= len; i++) {
        for (int prv = 0; prv < i; prv++) {
            int common = min({i - prv, len - i, lcp[l + prv][l + i]});
            int result = dp[prv][i] + (i - prv - common) + 1;
            if (dp[i][i + common] > result) {
                dp[i][i + common] = result;
                back[i][i + common] = prv;
            }
        }

        for (int nxt = i; nxt < len; nxt++) {
            if (dp[i][nxt + 1] > dp[i][nxt] + 1) {
                dp[i][nxt + 1] = dp[i][nxt] + 1;
                back[i][nxt + 1] = back[i][nxt];
            }
        }

        for (int nxt = len; nxt > i; nxt--) {
            if (dp[i][nxt - 1] > dp[i][nxt] + 1) {
                dp[i][nxt - 1] = dp[i][nxt] + 1;
                back[i][nxt - 1] = back[i][nxt];
            }
        }
    }

    return {dp, back};
}

pair<int, vector<int>> trace(dp_t const &dp, dp_t const &back, int len) {
    int answer = len + 1;
    int prv = 0;

    for (int i = 1; i < len; i++) {
        if (answer > dp[i][len]) {
            answer = dp[i][len];
            prv = i;
        }
    }

    int cur = len;
    vector<int> segments{cur, prv};
    while (prv != 0) {
        int tmp = prv;
        prv = back[prv][cur];
        cur = tmp;
        segments.push_back(prv);
    }

    reverse(all(segments));
    return {answer, segments};
}

void restore(vector<int> const &segments, vector<int> &actions, int l, int skip = 0) {
    vector<int> stk;
    int t = 0;
    for (int i = 1; i < segments.size(); i++) {
        int prv = segments[i - 1], cur = segments[i];
        while (stk.size() > cur - prv || i > 1 && stk.size() > lcp[l + prv][l + segments[i - 2]]) {
            actions.push_back(-1);
            stk.pop_back();
        }
        while (stk.size() < cur - prv) {
            int item = a[l + prv + stk.size()];
            if (t++ >= skip) actions.push_back(item);
            stk.push_back(item);
        }
        actions.push_back(0);
    }
}

void print(vector<int> const &actions, int l, int r) {
    if (actions.size() > r - l + 1) {
        naive(l, r);
    } else {
        cout << actions.size() << '\n';
        for (int item : actions) cout << item << ' ';
        cout << '\n';
    }
}

void solve_equal() {
    auto [dp, back] = calc(0);

    for (int i = 0; i < q; i++) {
        in(int, l, r)
        l--;
        int len = r - l;
        auto [answer, segments] = trace(dp, back, len);
        vector<int> actions;
        restore(segments, actions, 0);
        print(actions, 0, len);
    }
}

void solve_divider() {
    int pos = max_element(all(a)) - a.begin();
    a[pos] = 1;
    auto [dp, back] = calc(0);
    a[pos] = 2;

    for (int i = 0; i < q; i++) {
        in(int, l, r)
        l--;
        int len = r - l;

        vector<int> actions;
        if (l > pos || r <= pos) {
            a[pos] = 1;
            auto [answer, segments] = trace(dp, back, len);
            restore(segments, actions, 0);
            a[pos] = 2;
        } else if (len <= THR) {
            auto [dp2, back2] = calc(l, r);
            auto [answer, segments] = trace(dp2, back2, r - l);
            restore(segments, actions, l);
        } else {
            int left = pos - l, right = r - pos - 1;
            if (left == 0) {
                actions.push_back(2);
                actions.push_back(0);
                actions.push_back(-1);

                if (right > 0) {
                    auto [ans_r, seg_r] = trace(dp, back, right);
                    restore(seg_r, actions, pos + 1);
                }
            } else {
                auto [ans_l, seg_l] = trace(dp, back, left);
                int last_l = left - seg_l[seg_l.size() - 2];
                restore(seg_l, actions, 0);

                assert(actions.back() == 0);
                actions.pop_back();
                actions.push_back(2);
                actions.push_back(0);
                actions.push_back(-1);

                if (right > 0) {
                    int cnt = (right + last_l - 1) / last_l;
                    int mn = right % last_l > 0 ? (right + cnt - 1) / cnt - 1 : last_l;
                    int result = (last_l - mn) + cnt, mx = mn;
                    if (cnt > 1) {
                        for (int j = (right + cnt - 2) / (cnt - 1); j <= right; j++) {
                            int ans_r = (j - last_l) + (right + j - 1) / j;
                            if (result > ans_r || j <= last_l && result >= ans_r) {
                                result = ans_r;
                                mx = j;
                            }
                        }
                    }

                    if (mx < last_l) {
                        right -= cnt * mx;
                        int delta = last_l - mx;
                        for (int j = 0; j < cnt; j++) {
                            while (right < delta) {
                                actions.push_back(-1);
                                delta--;
                            }
                            actions.push_back(0);
                            right -= delta;
                        }
                    } else {
                        cnt = (right + mx - 1) / mx;
                        right -= cnt * last_l;
                        int add = 0;
                        for (int j = 0; j < cnt; j++) {
                            int rem = cnt - j;
                            while (rem * (add + 1) <= right) {
                                actions.push_back(1);
                                add++;
                            }
                            actions.push_back(0);
                            right -= add;
                        }
                    }
                }
            }
        }
        print(actions, l, r);
    }
}

void solve_full_precalc() {
    dp_all.resize(n);
    for (int start = 0; start < n; start++) {
        dp_all[start] = calc(start);
    }

    for (int t = 0; t < q; t++) {
        in(int, l, r)
        l--;

        auto const &[dp, back] = dp_all[l];
        auto [answer, segments] = trace(dp, back, r - l);
        cout << answer << '\n';
        vector<int> stk;
        for (int i = 1; i < segments.size(); i++) {
            int prv = segments[i - 1], cur = segments[i];
            while (stk.size() > cur - prv || i > 1 && stk.size() > lcp[l + prv][l + segments[i - 2]]) {
                cout << "-1 ";
                stk.pop_back();
            }
            while (stk.size() < cur - prv) {
                int item = a[l + prv + stk.size()];
                cout << item << ' ';
                stk.push_back(item);
            }
            cout << "0 ";
        }
        cout << '\n';
    }
}

int main() {

    ios_base::sync_with_stdio(false);
    cin.tie(nullptr);
    cout.tie(nullptr);

    cin >> n >> q;
    in(int, pts, gamma)
    a.resize(n);
    cin >> a;

    lcp.resize(n + 1, vector<int>(n + 1, 0));
    for (int i = n - 1; i > -1; i--) {
        for (int j = n - 1; j > -1; j--) {
            lcp[i][j] = a[i] == a[j] ? lcp[i + 1][j + 1] + 1 : 0;
        }
    }

    if (*max_element(all(a)) == 1) {
        solve_equal();
        return 0;
    }
    if (*max_element(all(a)) == 2 && count(all(a), 2) == 1) {
        solve_divider();
        return 0;
    }
    if (n <= 200) {
        solve_full_precalc();
        return 0;
    }

    dp_all.resize(n / THR + 1);
    for (int start = 0; start < dp_all.size(); start++) {
        dp_all[start] = calc(start * THR);
    }
    for (int t = 0; t < q; t++) {
        in(int, l, r)
        l--;

        vector<int> actions;
        if (r - l <= THR || q == 1) {
            auto [dp, back] = calc(l, r);
            auto [answer, segments] = trace(dp, back, r - l);
            restore(segments, actions, l);
            assert(actions.size() == answer);
        } else {
            int m = (l + THR - 1) / THR, cur = m * THR, prv = m * THR;
            if (cur > l) {
                auto [dp, back] = calc(l, cur);
                auto [_, segments] = trace(dp, back, cur - l);
                restore(segments, actions, l);
                prv = l + segments[segments.size() - 2];
            }
            auto const &[dp, back] = dp_all[m];
            auto [_, segments] = trace(dp, back, r - cur);
            int skip = 0;
            if (cur > l) {
                skip = min({cur - prv, segments[1] - segments[0], lcp[cur][prv]});
                for (int i = 0; i < cur - prv - skip; i++) {
                    actions.push_back(-1);
                }
            }
            restore(segments, actions, cur, skip);
        }

        print(actions, l, r);
    }

}